// ------------------------------------------------------------------
// Copyright (C) 2005 INRA <eugene@ossau.toulouse.inra.fr>
//
// This program is open source; you can redistribute it and/or modify
// it under the terms of the Artistic License (see LICENSE file).
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//
// You should have received a copy of Artistic License along with
// this program; if not, please see http://www.opensource.org
//
// $Id$
// ------------------------------------------------------------------
// File:		 AltEst.hpp
// Contents: class Alternative Est
// ------------------------------------------------------------------

#ifndef ALTEST_H_INCLUDED
#define ALTEST_H_INCLUDED

/** \cond */ // Do not include standard depedencies in doxygen graph
#include <vector>
/** \endcond */

#include "Hits.hpp"
#include "SensorIF.hpp"

/*************************************************************
 **											 OneAltEst
 *************************************************************/
class OneAltEst{
	friend class AltEst;

	private:
		std::string id;
		int  start, end, index;
		char strand;
		int  exonsNumber, totalLength;
		bool altSplicingEvidence;
		bool toRemove;
		std::vector <int> vi_ExonStart;
		std::vector <int> vi_ExonEnd;
		void Penalize(int pos, DATA *Data, double altPenalty);
		Logger log;

	public:
		OneAltEst  ();
		OneAltEst  (const std::string& id, int i, int j, char s=0);
		~OneAltEst ();
		void Reset						 ();
		void AddExon					 (int, int);
		void RemoveExon				 (int  i);
		void ExtremitiesTrim	 (int  exonucleasicLength);
		int  IsFiltered				 (bool unspliced, bool extremelen, bool verbose,
					int  minIn,			int  maxIn,			 int	maxEx,	 int minEx,
					int  minEstLen, int  maxEstLen);
		bool IsInconsistentWith(OneAltEst*);
		void Print					 () const;
		inline void  UpdateBoundaries() { start = vi_ExonStart[0]; end = vi_ExonEnd[vi_ExonEnd.size()-1]; };
		inline std::string GetId() const { return id;  };
		inline int	 GetEnd() const { return end; };
		inline int GetStart() const {return start; };
		inline char GetStrand() const {return strand; }
		inline int	 GetAltSplE() const { return altSplicingEvidence; };
		inline void  PutAltSplE(bool b) { altSplicingEvidence = b;		};
		inline void  FlagToRemove(bool b) { toRemove = b; };
		inline int	 IsToRemove() const { return toRemove; };
		inline void PutIndex(int i) { index = i;};
		inline const std::vector<int>& get_exonStart () const { return vi_ExonStart; }
		inline const std::vector<int>& get_exonEnd () const { return vi_ExonEnd; }
		inline int get_exonsNumber() const { return exonsNumber; }
};

/*************************************************************
 **											 AltEst
 *************************************************************/
class AltEst{
	private:
		bool altEstDisplay,  verbose;
		int  minIn, maxIn, maxEx, minEx;
		int  minEstLength, maxEstLength, exonucleasicLength;

		double altPenalty;
		int includedEstFilter;
		int compatibleEstFilter;
		int unsplicedEstFilter;
		int extremeLengthFilter;
		int nextAdd, nextRemove;

		int  ReadAltFile ( const std::string&, int &nbUnspliced, int &nbExtremLen);	// never used
		void Compare		 (int &nbIncomp, int &nbNoevidence, int &nbIncluded);

	public:
		int  totalAltEstNumber;
		int  keptAltEstNumber;
		std::vector<OneAltEst> voae_AltEst;

		AltEst	(DNASeq *X);
		~AltEst ();
		void Penalize(int i, int pos, DATA *Data);
		int convertHitsToAltEst (Hits * AllEST, int &nbUnspliced, int &nbExtremLen, int & NumEST);
};

#endif
