// ------------------------------------------------------------------
// Copyright (C) 2004 INRA <eugene@ossau.toulouse.inra.fr>
//
// This program is open source; you can redistribute it and/or modify
// it under the terms of the Artistic License (see LICENSE file).
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
//
// You should have received a copy of Artistic License along with
// this program; if not, please see http://www.opensource.org
//
// $Id$
// ------------------------------------------------------------------
// File:     DAG.h
// Contents: Modeling genes as a DAG and exploring it - Header
// ------------------------------------------------------------------

#ifndef DAG_H_INCLUDED
#define DAG_H_INCLUDED

/** \cond */ // Do not include standard depedencies in doxygen graph
#include <array>
#include <vector>
/** \endcond */

#include "BackP.hpp"
 
class DAG {
	private:
		DNASeq* seq;
		double normalizing_path;
		double PBest[NbTracks];
		BackPoint* PrevBP[NbTracks];

		const std::vector<short int>& activeTracks;
		const std::vector<bool>& isActiveTrack;

		int StartPosition;
		int EndPosition;
		Track  LBP[NbTracks];
  	   
		void Normalize();
		void ComputeSigShifts(enum Signal::Edge Strand, DATA Data, int position);
		void ComputeRequired(Signal::Edge, DATA, int);
		void ApplyScore(int position, DATA Data, int NoContentsUpdate);
		void ApplyLengthPenalty(int position, DATA Data, int NoContentsUpdate);
		double SuperMix(double a, double b);

		inline int GetStart() { return StartPosition; };
		void StatActive();
		void StatGC();

	public:
		DAG (int start, int end, DNASeq *Seq, const std::vector<short int>& activeTracks, const std::vector<bool>& isActiveTrack );
		~DAG ();
		void WeightThePrior();
		void LoadDistLength();
		void ShortestPathAlgoForward (int position, DATA Data);
		void ShortestPathAlgoBackward (int position, DATA Data, int NoContentsUpdate=0);
		void MarkAndSweep(int,int,int);
		Prediction* BuildPrediction (int From, int To, int Forward = 1);

		/** Write the DAG structure into a TSV file
		 * \param fileprefix The filename of the file to write, extension '.dag.tsv' is added to it
		*/
		void print_structure( const std::string& fileprefix ) const; // Debug-print the DAG contents
};

#endif // DAG_H_INCLUDED
