
#ifndef GENEFEATURE_H
#define GENEFEATURE_H

/** \cond */ // Do not include standard depedencies in doxygen graph
#include <string>
#include <iostream>
/** \endcond */

//Feature class:
#include "Locus.hpp"
#include "Attributes.hpp"

/**
 * class GeneFeature
 */


class GeneFeature
{
  private:
    std::string    id_;
    std::string    seqid_;
    std::string    source_;
    std::string    type_;
    Locus *   locus_;
    int       length_;
    double    score_;
    char      phase_;
    Attributes * attributes_;
    bool      valid_ ;
    
    void      parseLine( const std::string& line, int );
    
  public:
  
    // Constructors/Destructors
    GeneFeature ( );
    GeneFeature ( const char * const , int); 
    GeneFeature ( const GeneFeature & gene) ;
    //GeneFeature (std::string seqId, std::string source, std::string type, int start, int end, double score, char strand, char phase, char * attributes);
    virtual ~GeneFeature ( );
    
    void    setValid ( bool valid);
    void    setSeqId ( std::string id );
    
    bool    getValid ( )  const ;
    std::string  getSeqId ( )  const ;
    std::string  getString ( ) const ;
    char    getPhase ( )  const ;
    std::string  getId ( )     const ;
    std::string  getParent ( ) const ;
    std::string  getType ( )   const ;
    Locus * getLocus ( )  const ;
    double  getScore ( )  const ;
    int     getLength ( ) const ;
    bool    hasTarget ( )  const ;
    Attributes * getAttributes () const ;
    
    friend std::ostream& operator<<(std::ostream& o, const GeneFeature & geneFeature )
    {
      return o <<  geneFeature.getString();
    }

};

#endif // GENEFEATURE_H
