// ------------------------------------------------------------------
// Copyright (C) 2004 INRA <eugene@ossau.toulouse.inra.fr>
//
// This program is open source; you can redistribute it and/or modify
// it under the terms of the Artistic License (see LICENSE file).
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
//
// You should have received a copy of Artistic License along with
// this program; if not, please see http://www.opensource.org
//
// $Id$
// ------------------------------------------------------------------
// File:		 Hits.hpp
// Contents: Definitions for a class representing alignements with genomic seq.
// ------------------------------------------------------------------

#ifndef  HITS_H_INCLUDED
#define  HITS_H_INCLUDED

/** \cond */ // Do not include standard depedencies in doxygen graph
#include <stdio.h>
#include <string>
/** \endcond */
#include "GeneFeatureSet.hpp"
#include "DNASeq.hpp"

class Block{
	public:
		Block ();
		Block(size_t Start, size_t End, size_t LStart, size_t LEnd, int Ph, int Scr);

		~Block ();

		void AddBlockAfter(size_t Start, size_t End, size_t LStart, size_t LEnd,int Ph,int Scr);
		void AddBlockAfter(Block* newBlock);
		size_t Start;
		size_t End;
		int Phase;
		int Score;
		size_t LStart;
		size_t LEnd;
		Block* Prev;
		Block* Next;
};

class  Hits {
	public:
		Hits ();
		Hits (const std::string& name, int length, char strand, size_t deb, size_t fin, size_t ldeb, size_t lfin, int Ph, int Scr, double Prob, int level, int sup);
		Hits* ReadFromFile(FILE* HitFile, int *NumHits, int level, int margin, size_t maxPos);
		Hits* ReadFromGeneFeatureSet(GeneFeatureSet& HitSet , int *NumHits, int level, int margin, DNASeq *X);
		~Hits ();

		std::string Name;
		char	 Strand;
		int		 Length;
		int		 NGaps;
		int		 Start;
		int		 End;
		double Evalue;
		char	 Rejected;
		int		 Level;					// For blastx level 0-9
		int		 Support;

		Block* Match;
		Hits* Next;

		const std::string& getName () const { return this->Name; }
};
#endif
