// ------------------------------------------------------------------
// Copyright (C) 2004 INRA <eugene@ossau.toulouse.inra.fr>
//
// This program is open source; you can redistribute it and/or modify
// it under the terms of the Artistic License (see LICENSE file).
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
//
// You should have received a copy of Artistic License along with
// this program; if not, please see http://www.opensource.org
//
// $Id$
// ------------------------------------------------------------------
// File:		 Sensor.h
// Contents: class Sensor
// ------------------------------------------------------------------

#ifndef  SENSOR_H_INCLUDED
#define  SENSOR_H_INCLUDED

/** \cond */ // Do not include standard depedencies in doxygen graph
#include <vector>
#include <algorithm>
#include <string>
/** \endcond */

#include "GDIF/gdIF.h" // At the moment sensors must be plot using this one, in the future let each one plot it's own thing ?
#include "DNASeq.hpp"
#include "Param.hpp" // May be require by sensors (and plugins-sensors), so do not remove
#include "SensorIF.hpp"


// Avoid Cyclic
class Prediction;
class MasterSensor;

/*************************************************************
 **												 Sensor														**
 *************************************************************/
class Sensor {
	private:
		int instanceNumber;

	protected:
		Logger log;
		void CheckStart		(DNASeq *,std::vector<int> a, std::vector<int> b);
		void CheckStop		(DNASeq *,std::vector<int> a, std::vector<int> b);
		void CheckSplices (DNASeq *,std::vector<int> a, std::vector<int> b, std::vector<int> c, std::vector<int> d);
		void PlotStop			(int pos,		int phase,	int sure);
		void PlotStart		(int pos,		int phase,	double strength);
		void PlotAcc			(int pos,		int strand, double strength);
		void PlotDon			(int pos,		int strand, double strength);
		void PlotESTHit		(int start, int end,		int strand, int filtered);
		void PlotESTGap		(int start, int end,		int strand, int filtered);
		void PlotBlastHit (int start, int end,		int phase,	int level);
		void PlotBlastGap (int start, int phase1, int end,		int phase2, int level);
		void PlotStartReg (int pos,		int phase,	int color);
		void PlotEndReg		(int pos,		int phase,	int color);
		void PlotRepeat		(int start, int end);
		int  GetNumber		(void) { return instanceNumber; }
		//CN
		std::string inputFormat_;
		MasterSensor* MS = nullptr; // The Master of this sensor, should be manage with set_master and unset_master and never manually modified
	public:
		unsigned char type;
		/** Build a Sensor
		 * \param n	the instance ID of the sensor
		 * \param logs_name the name to use in logs output (be short and clear, avoid spaces for clarity)
		 * \param ms	the MasterSensor that created and manages this sensor
		 */
		Sensor	(int n, std::string logs_name, MasterSensor* ms = nullptr );
		virtual ~Sensor (void);
		virtual void Init				(DNASeq *) = 0;
		virtual void GiveInfo		(DNASeq *, size_t, DATA *) = 0;
		virtual void Plot				(DNASeq *){ return; } // Do nothing by default
		virtual void PostAnalyse( const Prediction * const, FILE *){ return; }; // DO nothing by default, Sensors specficying this one may need to include Prediction;hpp in their source file

		void set_master( MasterSensor* ms ){ this->MS = ms; }
		void unset_master(){ this->MS = nullptr; }
		bool isset_master(){ return( this->MS != nullptr ); }
};

/*************************************************************
 **												Signals Object										**
 *************************************************************/
class Signals {
	public:
		size_t		pos;
		int		type;
		int		edge;
		char	*score;

		Signals  ();
		Signals  (size_t, int, int, char*);
		~Signals ();
		void PrintS ();
		bool operator < (size_t i) { if (pos < i) return true; else return false; }
};

/*************************************************************
 **											 Contents Object										**
 *************************************************************/
class Contents {
	public:
		size_t start;
		size_t end;
		int		 type;
		float *score;

		Contents	();
		Contents	(size_t, size_t, int, float*);
		~Contents ();
		void PrintC ();
};


#endif

