#include "logger.hpp"

/** \cond */ // Do not include standard depedencies in doxygen graph
#include <iomanip> // for setprecision
#include <iostream>
#include <sstream> // for stringstream
#include <string>
/** \endcond */

using namespace std;

Logger::Logger( const string& subject ){
	this->subject = subject;
}

void Logger::log_message( const string& message, const string& level ) const {
	// Prefixes for message level and subject
	bool with_prefix=false;
	if( level.length() ){
		cerr << "[" << level << "]";
		with_prefix = true;
	}
	if( this->subject.length() ){
		cerr << "{" << this->subject << "}";
		with_prefix = true;
	}

	if( with_prefix ){ cerr << " "; } // Add a space between prefix to increase readability of message
	cerr << message << endl;
	return;
}

void Logger::error( const string& message ) const { this->log_message( message, "ERROR" ); }

void Logger::warning( const string& message ) const { this->log_message( message, "WARNING" ); }

void Logger::info( const string& message ) const { this->log_message( message, "INFO" ); }

#ifdef DEBUG_MODE
void Logger::debug( const string& message ) const { this->log_message( message, "DEBUG" ); }
#else
void Logger::debug( const string& ) const {}
#endif

// Static method
string Logger::format( const double number, const unsigned int precision ){
	stringstream ss;
	// 'fixed' is a format, like 'scientific', 'hexfloat', 'defaultfloat'
	// See: https://en.cppreference.com/W/cpp/header/ios
	ss << fixed << setprecision(precision) << number;
	return ss.str();
}