#ifndef EUGENE_LOGGER_HPP
#define EUGENE_LOGGER_HPP

/** \cond */ // Do not include standard depedencies in doxygen graph
#include <string>
/** \endcond */

/**
 * keeping in mind that POSIC **stdout is for results**
 * and stderr is for informations and errors: **all logs go into stderr**
 * 
 * Note: debug messages will be done only if compilation was set with DEBUG_MODE directive (it's defined)
*/

class Logger {
	public:
		Logger( const std::string& subject = "" );

		/** Log an error message */
		void error( const std::string& message ) const;
		/** Log a warning message */
		void warning( const std::string& message ) const;
		/** Log an info message */
		void info( const std::string& message ) const;
		/** Log a debug message, only if **DEBUG_MODE** directive was defined at compile-time */
		void debug( const std::string& message ) const;

		std::string get_subject() const { return this->subject; }
		void set_subject( const std::string& subject ){ this->subject = subject; }
		void clear_subject(){ this->subject.clear(); }


		// Format a floating number to string with a specific precision
		static std::string format( const double number, const unsigned int precision = 2 );

	private:
		std::string subject;

		void log_message( const std::string& message, const std::string& level ) const;
};
#endif