// ------------------------------------------------------------------
// Copyright (C) 2004 INRA <eugene@ossau.toulouse.inra.fr>
//
// This program is open source; you can redistribute it and/or modify
// it under the terms of the Artistic License (see LICENSE file).
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
//
// You should have received a copy of Artistic License along with
// this program; if not, please see http://www.opensource.org
//
// $Id$
// ------------------------------------------------------------------
// File:     Sensor.EuStop.cc
// Contents: Sensor EuStop
// ------------------------------------------------------------------

#include "EuStop.hpp"

extern Parameters PAR;

/*************************************************************
 **                      SensorEuStop                       **
 *************************************************************/
namespace {
  const bool is_declared = MasterSensor::declare_sensor( "EuStop", &(SensorEuStop::constructor) );
}
Sensor* SensorEuStop::constructor( int n, DNASeq* X, MasterSensor* ){ return new SensorEuStop( n, X ); }

// ----------------------
//  Default constructor.
// ----------------------
SensorEuStop :: SensorEuStop (int n, DNASeq */*X*/) : Sensor(n, "EuStop")
{
  type = Type_Stop;
}

// ----------------------
//  Default destructor.
// ----------------------
SensorEuStop :: ~SensorEuStop ()
{
}

// ----------------------
//  Init stop.
// ----------------------
void SensorEuStop :: Init (DNASeq *X)
{
  stopP = PAR.getD("EuStop.stopP*");
  
  if (PAR.getI("Output.graph")) Plot(X);
}

// -----------------------
//  GiveInfo signal stop.
// -----------------------
void SensorEuStop :: GiveInfo (DNASeq *X, size_t pos, DATA *d)
{
  double theStop;
  
  if ((theStop = X->IsStop(pos-3,1)) != 0.0) {
    d->sig[DATA::Stop].weight[Signal::Forward] += -stopP+std::log(theStop);
    d->sig[DATA::Stop].weight[Signal::ForwardNo] += std::log(1.0-theStop); 
  }
      
  if ((theStop = X->IsStop(pos+2,-1)) != 0.0) {
    d->sig[DATA::Stop].weight[Signal::Reverse] += -stopP+std::log(theStop);
    d->sig[DATA::Stop].weight[Signal::ReverseNo] += std::log(1.0-theStop); 
  }
}

// ----------------------------
//  Plot Sensor information
// ----------------------------
void SensorEuStop :: Plot(DNASeq *X)
{
  double Strength;

  for(size_t i = 0;  i <= X->SeqLen;  i++) {
    if ((Strength= X->IsStop(i-3,1)) != 0.0) 
      PlotStop(i,(i%3)+1,(Strength == 1.0));

    if((Strength = X->IsStop(i+2,-1)) != 0.0) 
      PlotStop(i,-((X->SeqLen-i)%3)-1,(Strength == 1.0));
  }
}
