// ------------------------------------------------------------------
// Copyright (C) 2014 INRA <eugene-help@lists.mulcyber.toulouse.inra.fr>
//
// This program is open source; you can redistribute it and/or modify
// it under the terms of the Artistic License (see LICENSE file).
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
//
// You should have received a copy of Artistic License along with
// this program; if not, please see http://www.opensource.org
//
// $Id$
// ------------------------------------------------------------------
// File:     Sensor.SignalWAM.h
// Contents: Sensor SignalWAM
// A simple signal detection sensor based on a Weight Array Model
// ------------------------------------------------------------------


#ifndef  SENSOR_SIGNALWAM_H_INCLUDED
#define  SENSOR_SIGNALWAM_H_INCLUDED

#include "../../MSensor.hpp"
#include "../../0_SensorTk/WAM.h"

/*************************************************************
 **                 SensorSignalWAM                      **
 *************************************************************/
class SensorSignalWAM : public Sensor
{
 private:
  WAM* WAModel;
  size_t upstreamLen;   // nb of nt in the motif before the signal
  size_t downstreamLen; // nb of nt in the motif after the signal
  int sigType;          // type of the signal DATA::Start, DATA::Acc or DATA::Don
  char*  pattern;       // pattern of the signal. Example : 'GT' for a donor signal
  
  double scaleCoef;     // coef for the WAM score
  double scalePenalty;  //  penalty for the WAM score (score= ScaleCoef * WAMscore - ScalePenalty
  
  std::vector<size_t>    vPosF, vPosR;
  std::vector<double> vScoreF, vScoreR;
  size_t indexF, indexR;
  size_t positionGiveInfo;
  
 size_t motifLength;       // length of the motif (signal+ flanking regions)
 size_t newStatePos;
 int scoringMode;       // if mode=0, score the full motif
                        // if mode=1, score the coding site of the motif only
 size_t scoringMotifStart; // start position in the motif to compute the score
 size_t scoringMotifEnd;   // end position in the motif to compute the score
 
 bool allowedNonCanSite; // true to search non canonic spliced site
                        
 void AssertPatternLength(char*, int, int, char*);
 void SearchSignal(DNASeq *X);
 
 public:
  SensorSignalWAM  (int n, DNASeq *X);
  virtual ~SensorSignalWAM  () override;
  static Sensor* constructor( int n, DNASeq* X, MasterSensor* ){ return new SensorSignalWAM( n, X ); }
  virtual void Init       (DNASeq *) override;
  virtual void GiveInfo   (DNASeq *, size_t, DATA *) override;
  virtual void Plot       (DNASeq *) override;

};

#endif
