#! /usr/bin/env python3

import argparse
import csv
import graphviz
from pathlib import Path

__version__ = "v1.0.0"
__author__ = "Axel VERDIER"
__organization__ = "INRAE LIPME BIOINFO team"
__contact__ = "axel.verdier@inrae.fr"

def get_parameters() -> dict:
	parser = argparse.ArgumentParser(
				prog = "dag_tsv_to_svg",
				description = "Description TODO",
				epilog = '; '.join( [f"Version: {__version__}", f"Author: {__author__} ({__organization__})" ,f"Contact: {__contact__}" ] ),
				formatter_class = argparse.ArgumentDefaultsHelpFormatter
			)
	parser.add_argument( "input_file", help = "The tsv file to parse" )

	args = parser.parse_args()
	return( {
		"input_file": Path( args.input_file ).resolve()
	} )

expected_header = ["track","backpoint","next","prev","origin"]
if __name__ == "__main__":
	params = get_parameters()
	graph = graphviz.Digraph()

	graph.attr( 'node', label="" )

	tracks_first = {}

	print( "Loading DAG nodes" )
	with open( params["input_file"], 'r' ) as f:
		reader = csv.DictReader( f, delimiter="\t" ) # fieldnames =  )
		if reader.fieldnames != expected_header:
			print( "Got columns: " + str(reader.fieldnames) + " but expected " + str(expected_header) )
			exit(1)

		for row in reader:
			# First node of a track ? It's a new track
			if row["next"] == "TRACK":
				track_node = "track " + row["track"]
				graph.node( track_node, label = track_node )
				graph.edge( track_node, row["backpoint"] )
				tracks_first[ row["track"] ] = row["backpoint"]

			# Their are backpoints in EuGene, so Prev are in reality the next position
			# and avoid cyclic back
			if row[ "prev" ] != tracks_first[ row["track"] ]:
				graph.edge(
					row["backpoint"],
					row["prev"]
				)

				if row["origin"] not in [ row["backpoint"], "TRACK" ]:
					graph.edge(
						row["origin"],
						row["backpoint"]
					)

	# Render the graph
	# Alternative (faster): sfdp -x -Tpng TEST > TEST.png
	# to try the Alternative with svg see it's faster too
	print( "Rendering graph" )
	graph.format = "svg"
	graph.render( "./TEST", cleanup = True )
